# Manage time-based snapshots of the set of APT repositories Tails needs,
# in a reprepro setup.
#
# Note: the ensure parameter is not fully supported, because some resources
# we are managing here don't provide this functionality.
class tails::reprepro::snapshots::time_based (
  $architectures     = $tails::reprepro::snapshots::params::architectures,
  $automatic_refresh = present,
  $ensure            = present,
  $homedir           = '/srv/apt-snapshots/time-based',
  $signwith          = $tails::reprepro::snapshots::params::signwith,
  $user              = 'reprepro-time-based-snapshots',
) inherits tails::reprepro::snapshots::params {

  ### Sanity checks

  validate_hash($architectures)
  validate_re($automatic_refresh, '^(present|absent)$')
  validate_re($ensure, '^(present|absent)$')

  ### Resources

  $repositories_dir = "${homedir}/repositories"

  $package_ensure = $ensure ? {
    absent  => absent,
    default => present,
  }

  tails::reprepro::snapshots::base { 'time-based':
    ensure  => $ensure,
    homedir => $homedir,
    repositories_dir => $repositories_dir,
    user    => $user,
  }

  tails::reprepro::snapshots::time_based::repository { 'debian':
    ensure                        => $ensure,
    architectures                 => $architectures,
    automatic_refresh             => $automatic_refresh,
    automatic_refresh_delta_hours => 0,
    basedir                       => "${repositories_dir}/debian",
    signwith                      => $signwith,
    user                          => $user,
  }

  tails::reprepro::snapshots::time_based::repository { 'debian-security':
    ensure                        => $ensure,
    architectures                 => $architectures,
    automatic_refresh             => $automatic_refresh,
    automatic_refresh_delta_hours => 1,
    basedir                       => "${repositories_dir}/debian-security",
    signwith                      => $signwith,
    user                          => $user,
  }

  tails::reprepro::snapshots::time_based::repository { 'tails':
    ensure                        => $ensure,
    architectures                 => $architectures,
    automatic_refresh             => $automatic_refresh,
    automatic_refresh_delta_hours => 2,
    basedir                       => "${repositories_dir}/tails",
    signwith                      => $signwith,
    user                          => $user,
  }

  tails::reprepro::snapshots::time_based::repository { 'torproject':
    ensure                        => $ensure,
    architectures                 => $architectures,
    automatic_refresh             => $automatic_refresh,
    automatic_refresh_delta_hours => 3,
    basedir                       => "${repositories_dir}/torproject",
    signwith                      => $signwith,
    user                          => $user,
  }

  $tails_update_time_based_apt_snapshots_pkg_deps = [
    libcarp-assert-perl,
    libcarp-assert-more-perl,
    libipc-system-simple-perl,
    libpath-tiny-perl,
    libtry-tiny-perl,
  ]
  ensure_packages(
    $tails_update_time_based_apt_snapshots_pkg_deps,
    {'ensure' => $package_ensure}
  )
  file { '/usr/local/bin/tails-update-time-based-apt-snapshots':
    ensure  => $ensure,
    owner   => root,
    group   => root,
    mode    => '0755',
    source  => 'puppet:///modules/tails/reprepro/snapshots/time_based/tails-update-time-based-apt-snapshots',
    require => [
      Package[$tails_update_time_based_apt_snapshots_pkg_deps],
      Exec[
        'tails-reprepro-snapshots-time_based-import-upstream-keys-apt-keys.d',
        'tails-reprepro-snapshots-time_based-import-upstream-keys-upstream-keys.d'
      ],
    ],
  }

  # To avoid having to maintain them in yet another place, let's reuse
  # the keys we give to APT on our systems:
  tails::reprepro::snapshots::time_based::import_upstream_keys {
    'apt-keys.d':
      ensure  => $ensure,
      source  => 'puppet:///modules/tails/apt/keys.d',
      user    => $user,
      homedir => $homedir,
  }
  # These are the additional upstream keys we don't use elsewhere:
  tails::reprepro::snapshots::time_based::import_upstream_keys {
    'upstream-keys.d':
      ensure  => $ensure,
      source  => 'puppet:///modules/tails/reprepro/snapshots/time_based/upstream-keys.d',
      user    => $user,
      homedir => $homedir,
  }

  # XXX: missing bits
  # - import OpenPGP signing key

}
