# Manage a cronjob that updates time-based snapshots of APT repositories
# in a reprepro setup.
#
# This defined resource is meant to be declared up to 4 times on a given system.
# Its $id parameter must be an integer among [1..4].
#
# The serial number of the generated snapshots uses the configured system
# timezone, which should be set to UTC for better consistency with other date
# information embedded e.g. in the mirrored APT repositories.
define tails::reprepro::snapshots::time_based::cronjob (
  $repository,
  $basedir,
  $id,
  $delta_hours = 0,
  $ensure = present,
) {

  ### Sanity checks

  validate_re($id, '^[1-4]$')
  validate_re($delta_hours, '^[0-9]+$')
  validate_re($ensure, '^(present|absent)$')
  validate_re($repository, '^[a-zA-Z-]+$')
  validate_absolute_path($basedir)

  ### Resources

  cron { "tails-update-time-based-apt-snapshots-${repository}-${id}":
    ensure  => $ensure,
    command => "SILENT=1 PATH=\"/usr/local/bin:\$PATH\" flock -n '/var/lock/tails-update-time-based-apt-snapshots-${repository}' tails-update-time-based-apt-snapshots '${repository}' '${basedir}' \"$(date '+\%Y\%m\%d')0${id}\"",
    user    => 'reprepro-time-based-snapshots',
    hour    => ($delta_hours + 6 * ($id - 1)) % 24,
    minute  => 41,
    require => [
      File['/usr/local/bin/tails-update-time-based-apt-snapshots'],
      Reprepro::Repository["tails-time-based-snapshots-${repository}"],
    ],
  }

}
