# Manage time-based snapshots of one APT repository in a reprepro setup.
#
# This resource is meant to be used from tails::reprepro::snapshots::time_based,
# and is not designed to be used in isolation.
#
# Resource name: the upstream repository we are mirroring.
#
# Note: the ensure parameter is not fully supported, because some resources
# we are managing here don't provide this functionality.
define tails::reprepro::snapshots::time_based::repository (
  $architectures,
  $basedir,
  $signwith,
  $user,
  $ensure = present,
  $automatic_refresh = present,
  $automatic_refresh_delta_hours = 0,
  $distributions_template = "tails/reprepro/snapshots/time_based/${name}/distributions.erb",
  $ensure = present,
  $updates_template = "tails/reprepro/snapshots/time_based/${name}/updates.erb",
) {

  ### Sanity checks

  private()
  validate_hash($architectures)
  validate_re($name, '^[a-zA-Z-]+$')
  validate_absolute_path($basedir)
  validate_string($user)
  validate_re($automatic_refresh, '^(present|absent)$')
  validate_re($distributions_template, '^\S+$')
  validate_re($ensure, '^(present|absent)$')
  validate_re($updates_template, '^\S+$')

  ### Resources

  $repository = $name

  reprepro::repository { "tails-time-based-snapshots-${repository}":
    architectures                => $architectures,
    user                         => $user,
    group                        => $user,
    uploaders                    => [],
    basedir                      => $basedir,
    origin                       => "Time-based snapshots of ${repository} APT repository",
    basedir_mode                 => '0751',
    incoming_mode                => '0700',
    signwith                     => $signwith,
    manage_distributions_conf    => true,
    distributions_template       => $distributions_template,
    manage_incoming_conf         => false,
    handle_incoming_with_inotify => false,
    manage_updates_conf          => true,
    updates_template             => $updates_template,
  }

  $cronjob_ensure = $ensure ? {
    present => $automatic_refresh,
    absent  => absent,
  }

  tails::reprepro::snapshots::time_based::cronjob { "${repository}-1":
    ensure      => $cronjob_ensure,
    delta_hours => $automatic_refresh_delta_hours,
    id          => 1,
    repository  => $repository,
    basedir     => $basedir,
  }
  tails::reprepro::snapshots::time_based::cronjob { "${repository}-2":
    ensure      => $automatic_refresh,
    delta_hours => $automatic_refresh_delta_hours,
    id          => 2,
    repository  => $repository,
    basedir     => $basedir,
  }
  tails::reprepro::snapshots::time_based::cronjob { "${repository}-3":
    ensure      => $cronjob_ensure,
    delta_hours => $automatic_refresh_delta_hours,
    id          => 3,
    repository  => $repository,
    basedir     => $basedir,
  }
  tails::reprepro::snapshots::time_based::cronjob { "${repository}-4":
    ensure      => $cronjob_ensure,
    delta_hours => $automatic_refresh_delta_hours,
    id          => 4,
    repository  => $repository,
    basedir     => $basedir,
  }

}
